function [data,labels,raw,data_orig_EEG] = perform_Laplacian(sourceFilename,badEEG)

% perform surface Laplacian for EEG
% Example:
% [data,labels,raw,data_orig_EEG] = perform_Laplacian('/home/jukkaupp/EEGdata/TD01.fif',{'EEG008' 'EEG009' 'EEG022'});

% inputs:
% sourceFilename - fif-file name (string)
% badEEG - bad EEG channels (cell array containing strings)
%
% outputs:
% data - struct containing EEG, MEG, and combined MEG+EEG in different
% fields after performing surface Laplacian for EEG
% labels - corresponding channel labels
% raw - fif-information file
% data_orig_EEG struct containing data without surface Laplacian

if nargin == 1
    badEEG = {'EEG001' 'EEG035'};
end

%warning off
%[sourceFilename,classNames,triggerChannels,delays,chanIndex] = data_info(5,5)

raw = fiff_setup_read_raw(sourceFilename);
Fs = double(raw.info.sfreq);

CM = [];

%CM.filename = sourceFilename;
%CM.infile = sourceFilename; 
CM.seuil_magn = 5e-12; % rejection threshold for magnetometers
CM.seuil_grad = 1e-10; %7e-11; % rejection threshold for gradiometers
CM.seuil_EEG = 180e-6; % rejection threshold for EEG

%--- Give bad EEG channels: EEG = ch# 311-370 (EEG001-EEG60) ----
% badEEG = {'EEG001' 'EEG035'}; % list EEG channels that won't be analyzed
CM.badEEG = badEEG;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters initialization
sensors = get_sensors(raw);

bad = find(strcmp(raw.info.ch_names,'bad'));
if isempty(bad)
    if ~isfield(CM,'seuil_grad')
        bad = zeros(1,to-from+1);
    end
else
    bad = fiff_read_raw_segment(raw,0,inf,bad);
end

% pick MEG signals
picksMEG = sensors.picksMEG;
if isfield(CM,'badEEG')
    picksEEG = fiff_pick_types(raw.info,0,1,0,[],CM.badEEG);
    dontpicksEEG = fiff_pick_channels(raw.info.ch_names,CM.badEEG);
    allEEGs = fiff_pick_types(raw.info,0,1,0,[]);
    [tmp,badEEGchan] = intersect(allEEGs,dontpicksEEG);
else
    picksEEG = fiff_pick_types(raw.info,0,1);
end
picksMEEG = [picksMEG picksEEG];
Nmeg = length(picksMEG);
Neeg = length(picksEEG);
Nmeeg = Nmeg + Neeg;
%Nref = length(CM.ref);
picksmagns = sensors.picksmagns;
picksgrads = sensors.picksgrads;
subpicksEEG = Nmeg+1:Nmeg+Neeg;
labels = raw.info.ch_names(picksMEEG);

% surface laplacian (scalp current density)
cfg = [];
cfg.dataset = sourceFilename;
cfg.channel = raw.info.ch_names(picksEEG);

data_ft = ft_preprocessing(cfg);
data = fiff_read_raw_segment(raw,0,inf,picksMEEG);

% put EEG data in a ft structure
data_ft.trial = {data(subpicksEEG,:)};
data_ft.time = {1:size(data,2)};
data_ft.sampleinfo = [1 size(data,2)];

data_orig = data;

% estimate the surface laplacian
cfg_surflap = [];
cfg_surflap.method = 'finite';
cfg_surflap.elec = data_ft.elec;
dataSL = ft_scalpcurrentdensity(cfg_surflap,data_ft);
data(subpicksEEG,:) = dataSL.trial{1};

subpicksMEG = setdiff(1:size(data,1),subpicksEEG);

data_EEG = data(subpicksEEG,:);
data_MEG = data(subpicksMEG,:);
data_MEEG = data;

label_EEG = labels(subpicksEEG);
label_MEG = labels(subpicksMEG);
label_MEEG = labels;

clear labels data

data.EEG = data_EEG;
data.MEG = data_MEG;
data.MEEG = data_MEEG;
labels.EEG = label_EEG;
labels.MEG = label_MEG;
labels.MEEG = label_MEEG;

data_orig_EEG = data_orig(subpicksEEG,:);


% for m=[1:8 10:13 15];
%   [sourceFilename,classNames,triggerChannels,delays,chanIndex,subj_prefix,session] = data_info(5,m);
%   badEEG = return_bad_eeg(subj_prefix,session);
%   [data,labels,st1,data_orig_EEG] = perform_Laplacian(sourceFilename,badEEG);
%   EEG = data.EEG;
%   EEG=EEG(:,100000:200000);
%   data_orig_EEG = data_orig_EEG(:,100000:200000);
%   save(['subj' num2str(m)],'EEG','data_orig_EEG');
%   disp(num2str(m));
% end
% close all
% sampl = 1:10000;
% load(['subj' num2str(1)],'EEG','data_orig_EEG');
% sampl = 1:size(EEG,2);
% for m=[1:8 10:13 15];
%     load(['subj' num2str(m)],'EEG','data_orig_EEG');
%     figure,
%     for s = 1:size(EEG,1)
%         E = EEG(s,:) - mean(EEG(s,:));
%         E = E/std(E);
%         EO = data_orig_EEG(s,:) - mean(data_orig_EEG(s,:));
%         EO = EO/std(EO);
%         %figure,plot(E(sampl));hold on;plot(EO(sampl),'r')
%         subplot(5,11,s)
%         plot(E(sampl),'r');hold on;plot(10+EO(sampl),'b');grid on;xlim([1 length(sampl)])
%         set(gca,'XTickLabel',[],'YTickLabel',[])
%     end
%     set(gcf,'Name',['subject: ' num2str(m) ': red = Laplacian, blue = original'])
%     saveas(gcf,['subject ' num2str(m) ' red is Laplacian and blue is original'],'jpg')
% end

