function features = project_data(data,U,V,proj_type)
% This function extracts features from data using linear projections. 
%
% Inputs:
%
% data - a data set to be projected. data is a C*N*F matrix, where:
% C = the number of ICs or channels
% N = the number of time-windows
% F = the number of frequency bins
%
% U,V - coefficient vectors, i.e., the directions of projections.
%
% Note! When Spectral LDA projection type is chosen, projections are stored
% only in "V". When Three-way DA projection type is chosen, projections are
% stored in both "U" and "V".
% 
% proj_type - feature extraction type used, must be either 
% "threeway_DA" or "spectral_DA".
% 
% Outputs:
%
% features - projected epochs. These projections will be used as 
% classifier inputs

switch proj_type
    case 'threeway_DA'        
        features = zeros(size(U,2),size(data,2))';
        for m = 1:size(data,2)
            Dat = squeeze(data(:,m,:));
            features(m,:) = diag(U'*Dat*V);
        end
    case 'spectral_LDA'        
        data = permute(data,[3 2 1]);        
        p = size(data,1);
        n = size(data,2);
        c = size(data,3);        
        nrFeatures = size(V,1); % number of estimated directions        
        dataProj = zeros(nrFeatures,c,n);
        iter = 1;
        for m = 1:nrFeatures
            for k = 1:c
                w = squeeze(V(m,:,k));
                if sum(w) == 0
                    w = randn(size(w));
                end
                w = w/norm(w);
                w = w(:);
                X = data(:,:,k)';
                Y = X*w;
                dataProj(m,k,:) = Y;
            end
        end        
        features = tensor_to_matrix(dataProj);
    case 'spatial_LDA'        
        p = size(data,1);
        n = size(data,2);
        c = size(data,3);        
        nrFeatures = size(V,1); % number of estimated directions        
        dataProj = zeros(nrFeatures,c,n);
        iter = 1;
        for m = 1:nrFeatures
            for k = 1:c
                w = squeeze(V(m,:,k));
                if sum(w) == 0
                    w = randn(size(w));
                end
                w = w/norm(w);
                w = w(:);
                X = data(:,:,k)';
                Y = X*w;
                dataProj(m,k,:) = Y;
            end
        end        
        features = tensor_to_matrix(dataProj);
    case {'spectral_ICA','temporal_ICA','temporal_PCA'}
        data = permute(data,[3 2 1]);        
        p = size(data,1);
        n = size(data,2);
        c = size(data,3);        
        nrFeatures = size(V{1},1); % number of estimated directions        
        siz = zeros(1,length(V));
        for mm = 1:length(V)
            siz(mm) = size(V{mm},1);
        end
        nrFeatures = min(siz);
        dataProj = zeros(nrFeatures,c,n);
        iter = 1;
        for m = 1:nrFeatures
            for k = 1:c
                w = V{k}(m,:);
                if sum(w) == 0
                    w = randn(size(w));
                end
                w = w/norm(w);
                w = w(:);
                if strcmp(proj_type,'temporal_ICA') | strcmp(proj_type,'temporal_PCA') 
                    X = data(:,:,k)';
                    % center data:
                    X = X - repmat(mean(X,1),size(X,1),1);
                    % project data:
                    Y = X*w;
                    dataProj(m,k,:) = Y;
                else
                    X = abs(data(:,:,k)');
                    X = X - repmat(mean(X,1),size(X,1),1);
                    Y = X*w;
                    Y2 = abs((data(:,:,k)')*w);
                    Y3 = real((data(:,:,k)')*w);
                    dataProj(m,k,:) = Y2;
                end
            end
        end        
        features = tensor_to_matrix(dataProj);                
    otherwise
        error('Wrong projection type!')
end