function [resu,Classifier_final] = train_and_test_three_way_DA(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,params_three_way_DA)
% Train, validate and test Three-way discriminant analysis classifier.
%
% Inputs: 
% data_train - training data
% data_validation - validation data
% data_test - test data
%
% Data format: Data is a tensor of size C*N*F, where:
% C is the total number of channels or independent components 
% N is the total number of epochs 
% F is the number of frequency bins in each epoch.
%
% labels_train - class labels for training data
% labels_validation - class labels for validation data
% labels_test - class labels for test data
%
% Class label format: labels are provided as a vector of length N 
% denoting class labels of each epoch. Categories are indexed as 
% 1,2,3,...,K, where K is the total number of categories.
%
% params_three_way_DA - struct containing classifier parameters:
% params_three_way_DA.modelRank     Rank of the classifier
%
% Outputs:
%
% resu is a struct containing results:
% res.acc           Accuracy for the test data
% res.labelsEstim   Estimated class labels
% res.labelsTrue    True class labels
%
% Classifier_final - struct containing classifier parameters described in
% GLMNET:
% 
% Classifier_final.a0      Intercept sequence of length length(Classifier_final.lambda).
% Classifier_final.beta    For "elnet" and "lognet" models, a nvars x length(lambda) matrix of coefficients. For "multnet", a list of nc such matrices, one for each class.
% Classifier_final.lambda  The actual sequence of lambda values used.
% Classifier_final.dev     The fraction of (null) deviance explained (for "elnet", this is the R-square).
% Classifier_final.nulldev Null deviance (per observation).
% Classifier_final.df      The number of nonzero coefficients for each value of lambda. For "multnet", this is the number of variables with a nonzero coefficient for any class.
% Classifier_final.dfmat   For "multnet" only. A matrix consisting of the number of nonzero coefficients per class.
% Classifier_final.dim     Dimension of coefficient matrix (ices).
% Classifier_final.npasses Total passes over the data summed over all lambda values.
% Classifier_final.offset  a logical variable indicating whether an offset was included in the model.
% Classifier_final.jerr    Error flag, for warnings and errors (largely for internal debugging).
% Classifier_final.class   Type of regression - internal usage.
% Classifier_final.call    a cell including the names of all the input variables in the parent environment.

if exist('glmnet.m') ~= 2
    error('Cannot proceed because GLMNET-package not found from Matlab path.')
end

modelRank = params_three_way_DA.modelRank; % rank of the model

% extract features:
[U,V] = estimate_projections(data_train,labels_train,'threeway_DA',modelRank);
features_train = project_data(data_train,U,V,'threeway_DA');
features_validation = project_data(data_validation,U,V,'threeway_DA');
features_test = project_data(data_test,U,V,'threeway_DA');

% train and validate classifiers for several hyperparameters:
options = glmnetSet;
options.standardize=0;
Classifier = train_sparse_linear(features_train,labels_train,options);
resu = test_sparse_linear(Classifier,features_validation,labels_validation);
validation_accuracy = resu.acc;

% obtain the best model according to the validation results:
[bestValid,bestValidInd] = max(validation_accuracy);

% train the final classifier using training + validation data:
options = glmnetSet;
options.standardize = 0;
options.lambda = Classifier.lambda;
Classifier_final = train_sparse_linear([features_train;features_validation],[labels_train labels_validation],options);

% test classifier using independent test data:
res = test_sparse_linear(Classifier_final,features_test,labels_test);

% save classification results:
resu.acc = res.acc(bestValidInd); % accuracy
resu.labelsTrue = res.labelsTrue; % true classes
resu.labelsEstim = res.labelsEstim(:,bestValidInd); % estimated classes


